@@ -1,5 +1,5 @@
-/* auditd-event.c -- 
- * Copyright 2004-08,2011,2013,2015-16,2018 Red Hat Inc.,Durham, North Carolina.
+/* auditd-event.c --
+ * Copyright 2004-08,2011,2013,2015-16,2018,2021 Red Hat Inc.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -42,6 +42,7 @@
 #include "libaudit.h"
 #include "private.h"
 #include "auparse.h"
+#include "auparse-idata.h"
 
 /* This is defined in auditd.c */
 extern volatile int stop;
@@ -56,7 +57,7 @@ static void do_space_left_action(int admin);
 static void do_disk_full_action(void);
 static void do_disk_error_action(const char *func, int err);
 static void fix_disk_permissions(void);
-static void check_excess_logs(void); 
+static void check_excess_logs(void);
 static void rotate_logs_now(void);
 static void rotate_logs(unsigned int num_logs, unsigned int keep_logs);
 static void shift_logs(void);
@@ -70,7 +71,7 @@ static void init_flush_thread(void);
 /* Local Data */
 static struct daemon_conf *config;
 static volatile int log_fd;
-static FILE *log_file;
+static FILE *log_file = NULL;
 static unsigned int disk_err_warning = 0;
 static int fs_space_warning = 0;
 static int fs_admin_space_warning = 0;
@@ -85,6 +86,7 @@ static pthread_t flush_thread;
 static pthread_mutex_t flush_lock;
 static pthread_cond_t do_flush;
 static volatile int flush;
+static auparse_state_t *au = NULL;
 
 /* Local definitions */
 #define FORMAT_BUF_LEN (MAX_AUDIT_MESSAGE_LENGTH + _POSIX_HOST_NAME_MAX)
@@ -116,7 +118,7 @@ void write_logging_state(FILE *f)
 				config->space_left);
 			fprintf(f, "admin_space_left setting %lu MB\n",
 				config->admin_space_left);
-		}		
+		}
 		fprintf(f, "logging suspended = %s\n",
 					logging_suspended ? "yes" : "no");
 		fprintf(f, "file system space action performed = %s\n",
@@ -130,17 +132,14 @@ void write_logging_state(FILE *f)
 
 void shutdown_events(void)
 {
-	/* Give it 5 seconds to clear the queue */
-	alarm(5);
-
-	// Nudge the flush thread
-	pthread_cond_signal(&do_flush);
-	pthread_join(flush_thread, NULL);
-
+	// We are no longer processing events, sync the disk and close up.
+	pthread_cancel(flush_thread);
 	free((void *)format_buf);
+	auparse_destroy_ext(au, AUPARSE_DESTROY_ALL);
+	if (log_fd >= 0)
+		fsync(log_fd);
 	if (log_file)
 		fclose(log_file);
-	auparse_destroy_ext(NULL, AUPARSE_DESTROY_ALL);
 }
 
 int init_event(struct daemon_conf *conf)
@@ -159,8 +158,8 @@ int init_event(struct daemon_conf *conf)
 		log_fd = 1; // stdout
 		log_file = fdopen(log_fd, "a");
 		if (log_file == NULL) {
-			audit_msg(LOG_ERR, 
-				"Error setting up stdout descriptor (%s)", 
+			audit_msg(LOG_ERR,
+				"Error setting up stdout descriptor (%s)",
 				strerror(errno));
 			return 1;
 		}
@@ -176,7 +175,8 @@ int init_event(struct daemon_conf *conf)
 	format_buf = (char *)malloc(FORMAT_BUF_LEN);
 	if (format_buf == NULL) {
 		audit_msg(LOG_ERR, "No memory for formatting, exiting");
-		fclose(log_file);
+		if (log_file)
+			fclose(log_file);
 		log_file = NULL;
 		return 1;
 	}
@@ -192,11 +192,12 @@ static void *flush_thread_main(void *arg)
 
 	/* This is a worker thread. Don't handle signals. */
 	sigemptyset(&sigs);
-	sigaddset(&sigs, SIGALRM);
 	sigaddset(&sigs, SIGTERM);
 	sigaddset(&sigs, SIGHUP);
 	sigaddset(&sigs, SIGUSR1);
 	sigaddset(&sigs, SIGUSR2);
+	sigaddset(&sigs, SIGCHLD);
+	sigaddset(&sigs, SIGCONT);
 	pthread_sigmask(SIG_SETMASK, &sigs, NULL);
 
 	while (!stop) {
@@ -213,7 +214,8 @@ static void *flush_thread_main(void *arg)
 		flush = 0;
 		pthread_mutex_unlock(&flush_lock);
 
-		fsync(log_fd);
+		if (log_fd >= 0)
+			fsync(log_fd);
 	}
 	return NULL;
 }
@@ -226,6 +228,7 @@ static void init_flush_thread(void)
 	pthread_cond_init(&do_flush, NULL);
 	flush = 0;
 	pthread_create(&flush_thread, NULL, flush_thread_main, NULL);
+	pthread_detach(flush_thread);
 }
 
 static void replace_event_msg(struct auditd_event *e, const char *buf)
@@ -237,7 +240,8 @@ static void replace_event_msg(struct auditd_event *e, const char *buf)
 			e->reply.message = strdup(buf);
 		else {
 			// If too big, we must truncate the event due to API
-			e->reply.message = strndup(buf, MAX_AUDIT_MESSAGE_LENGTH-1);
+			e->reply.message = strndup(buf,
+						  MAX_AUDIT_MESSAGE_LENGTH-1);
 			len = MAX_AUDIT_MESSAGE_LENGTH;
 		}
 		// For network originating events, len should be used
@@ -262,7 +266,7 @@ static const char *format_raw(const struct audit_reply *rep)
 		"node=%s type=DAEMON_ERR op=format-raw msg=NULL res=failed",
                                 config->node_name);
 		else
-	        	snprintf(format_buf, MAX_AUDIT_MESSAGE_LENGTH,
+			snprintf(format_buf, MAX_AUDIT_MESSAGE_LENGTH,
 			  "type=DAEMON_ERR op=format-raw msg=NULL res=failed");
 	} else {
 		int len, nlen;
@@ -270,7 +274,7 @@ static const char *format_raw(const struct audit_reply *rep)
 		char unknown[32];
 		type = audit_msg_type_to_name(rep->type);
 		if (type == NULL) {
-			snprintf(unknown, sizeof(unknown), 
+			snprintf(unknown, sizeof(unknown),
 				"UNKNOWN[%d]", rep->type);
 			type = unknown;
 		}
@@ -282,7 +286,7 @@ static const char *format_raw(const struct audit_reply *rep)
 			len = rep->len;
 		}
 
-		// Note: This can truncate messages if 
+		// Note: This can truncate messages if
 		// MAX_AUDIT_MESSAGE_LENGTH is too small
 		if (config->node_name_format != N_NONE)
 			nlen = snprintf(format_buf, FORMAT_BUF_LEN - 32,
@@ -294,9 +298,9 @@ static const char *format_raw(const struct audit_reply *rep)
 				"type=%s msg=%.*s", type, len, message);
 
 	        /* Replace \n with space so it looks nicer. */
-        	ptr = format_buf;
+		ptr = format_buf;
 	        while ((ptr = strchr(ptr, 0x0A)) != NULL)
-        	        *ptr = ' ';
+		        *ptr = ' ';
 
 		/* Trim trailing space off since it wastes space */
 		if (format_buf[nlen-1] == ' ')
@@ -338,7 +342,7 @@ static int add_simple_field(auparse_state_t *au, size_t len_left, int encode)
 	}
 	field_name[i] = 0;
 	nlen = i;
-	
+
 	// get the translated value
 	value = auparse_interpret_field(au);
 	if (value == NULL)
@@ -392,12 +396,11 @@ static const char *format_enrich(const struct audit_reply *rep)
 	    "node=%s type=DAEMON_ERR op=format-enriched msg=NULL res=failed",
                                 config->node_name);
 		else
-	        	snprintf(format_buf, MAX_AUDIT_MESSAGE_LENGTH,
+			snprintf(format_buf, MAX_AUDIT_MESSAGE_LENGTH,
 		    "type=DAEMON_ERR op=format-enriched msg=NULL res=failed");
 	} else {
-		int rc;
+		int rc, rtype;
 		size_t mlen, len;
-		auparse_state_t *au;
 		char *message;
 		// Do raw format to get event started
 		format_raw(rep);
@@ -415,16 +418,31 @@ static const char *format_enrich(const struct audit_reply *rep)
 		format_buf[mlen] = 0;
 
 		// init auparse
-		au = auparse_init(AUSOURCE_BUFFER, message);
 		if (au == NULL) {
-			free(message);
-			return format_buf;
-		}
-		auparse_set_escape_mode(au, AUPARSE_ESC_RAW);
+			au = auparse_init(AUSOURCE_BUFFER, message);
+			if (au == NULL) {
+				free(message);
+				return format_buf;
+			}
+			auparse_set_escape_mode(au, AUPARSE_ESC_RAW);
+			auparse_set_eoe_timeout(config->end_of_event_timeout);
+		} else
+			auparse_new_buffer(au, message, mlen+1);
 		sep_done = 0;
 
 		// Loop over all fields while possible to add field
 		rc = auparse_first_record(au);
+		rtype = auparse_get_type(au);
+		switch (rtype)
+		{	// Flush before adding to pickup new associations
+			case AUDIT_ADD_USER:
+			case AUDIT_ADD_GROUP:
+				_auparse_flush_caches();
+				break;
+			default:
+				break;
+		}
+
 		while (rc > 0 && len > MIN_SPACE_LEFT) {
 			// See what kind of field we have
 			size_t vlen;
@@ -452,7 +470,17 @@ static const char *format_enrich(const struct audit_reply *rep)
 			rc = auparse_next_field(au);
 		}
 
-		auparse_destroy_ext(au, AUPARSE_DESTROY_COMMON);
+		switch(rtype)
+		{	// Flush after modification to remove stale entries
+			case AUDIT_USER_MGMT:
+			case AUDIT_DEL_USER:
+			case AUDIT_DEL_GROUP:
+			case AUDIT_GRP_MGMT:
+				_auparse_flush_caches();
+				break;
+			default:
+				break;
+		}
 		free(message);
 	}
         return format_buf;
@@ -552,20 +580,21 @@ void handle_event(struct auditd_event *e)
 					rc = fflush_unlocked(log_file);
 				} while (rc < 0 && errno == EINTR);
 		                if (errno) {
-		                	if (errno == ENOSPC && 
+					if (errno == ENOSPC &&
 					     fs_space_left == 1) {
 					     fs_space_left = 0;
 					     do_disk_full_action();
-		        	        } else
+				        } else
 					     //EIO is only likely failure mode
-					     do_disk_error_action("flush", 
+					     do_disk_error_action("flush",
 						errno);
 				}
 
 				if (config->daemonize == D_BACKGROUND) {
 					if (config->flush == FT_INCREMENTAL) {
 						/* EIO is only likely failure */
-						if (fsync(log_fd) != 0) {
+						if (log_fd >= 0 &&
+							fsync(log_fd) != 0) {
 						     do_disk_error_action(
 							"fsync",
 							errno);
@@ -574,7 +603,8 @@ void handle_event(struct auditd_event *e)
 						pthread_mutex_lock(&flush_lock);
 						flush = 1;
 						pthread_cond_signal(&do_flush);
-						pthread_mutex_unlock(&flush_lock);
+						pthread_mutex_unlock(
+								   &flush_lock);
 					}
 				}
 			}
@@ -601,7 +631,7 @@ static void send_ack(const struct auditd_event *e, int ack_type,
 void resume_logging(void)
 {
 	audit_msg(LOG_NOTICE, "Audit daemon is attempting to resume logging.");
-	logging_suspended = 0; 
+	logging_suspended = 0;
 	fs_space_left = 1;
 
 	// User space action scripts cause fd to close
@@ -611,7 +641,7 @@ void resume_logging(void)
 		fix_disk_permissions();
 		if (open_audit_log()) {
 			int saved_errno = errno;
-			audit_msg(LOG_WARNING, 
+			audit_msg(LOG_WARNING,
 				"Could not reopen a log after resume logging");
 			logging_suspended = 1;
 			do_disk_error_action("resume", saved_errno);
@@ -662,7 +692,7 @@ static void write_to_log(const struct auditd_event *e)
 			log_size += rc;
 			check_log_file_size();
 			// Keep loose tabs on the free space
-			if ((log_size % 3) < 2)
+			if ((log_size % 8) < 3)
 				check_space_left();
 		}
 
@@ -693,6 +723,14 @@ static void check_log_file_size(void)
 			case SZ_SUSPEND:
 				audit_msg(LOG_ERR,
 		    "Audit daemon is suspending logging due to logfile size.");
+				// We need to close the file so that manual
+				// intervention can move or delete the file.
+				// We don't want to keep logging to a deleted
+				// file.
+				if (log_file)
+					fclose(log_file);
+				log_file = NULL;
+				log_fd = -1;
 				logging_suspended = 1;
 				break;
 			case SZ_ROTATE:
@@ -708,7 +746,7 @@ static void check_log_file_size(void)
 					shift_logs();
 				break;
 			default:
-				audit_msg(LOG_ALERT, 
+				audit_msg(LOG_ALERT,
   "Audit daemon log file is larger than max size and unknown action requested");
 				break;
 		}
@@ -720,6 +758,9 @@ static void check_space_left(void)
 	int rc;
 	struct statfs buf;
 
+	if (log_fd < 0)
+		return;
+
         rc = fstatfs(log_fd, &buf);
         if (rc == 0) {
 		if (buf.f_bavail < 5) {
@@ -730,7 +771,7 @@ static void check_space_left(void)
 			unsigned long blocks;
 			unsigned long block_size = buf.f_bsize;
 		        blocks = config->space_left * (MEGABYTE/block_size);
-        		if (buf.f_bavail < blocks) {
+			if (buf.f_bavail < blocks) {
 				if (fs_space_warning == 0) {
 					do_space_left_action(0);
 					// Allow unlimited rotation
@@ -744,7 +785,7 @@ static void check_space_left(void)
 				fs_space_warning = 0;
 			}
 		        blocks=config->admin_space_left * (MEGABYTE/block_size);
-        		if (buf.f_bavail < blocks) {
+			if (buf.f_bavail < blocks) {
 				if (fs_admin_space_warning == 0) {
 					do_space_left_action(1);
 					// Allow unlimited rotation
@@ -759,11 +800,11 @@ static void check_space_left(void)
 			}
 		}
 	}
-	else audit_msg(LOG_DEBUG, "fstatfs returned:%d, %s", rc, 
+	else audit_msg(LOG_DEBUG, "fstatfs returned:%d, %s", rc,
 			strerror(errno));
 }
 
-extern int sendmail(const char *subject, const char *content, 
+extern int sendmail(const char *subject, const char *content,
 	const char *mail_acct);
 static void do_space_left_action(int admin)
 {
@@ -779,7 +820,7 @@ static void do_space_left_action(int admin)
 		case FA_IGNORE:
 			break;
 		case FA_SYSLOG:
-			audit_msg(LOG_ALERT, 
+			audit_msg(LOG_ALERT,
 			    "Audit daemon is low on disk space for logging");
 			break;
 		case FA_ROTATE:
@@ -791,23 +832,24 @@ static void do_space_left_action(int admin)
 			break;
 		case FA_EMAIL:
 			if (admin == 0) {
-				sendmail("Audit Disk Space Alert", 
+				sendmail("Audit Disk Space Alert",
 				"The audit daemon is low on disk space for logging! Please take action\nto ensure no loss of service.",
 					config->action_mail_acct);
-				audit_msg(LOG_ALERT, 
+				audit_msg(LOG_ALERT,
 			    "Audit daemon is low on disk space for logging");
 			} else {
-				sendmail("Audit Admin Space Alert", 
+				sendmail("Audit Admin Space Alert",
 				"The audit daemon is very low on disk space for logging! Immediate action\nis required to ensure no loss of service.",
 					config->action_mail_acct);
-				audit_msg(LOG_ALERT, 
+				audit_msg(LOG_ALERT,
 			  "Audit daemon is very low on disk space for logging");
 			}
 			break;
 		case FA_EXEC:
 			// Close the logging file in case the script zips or
 			// moves the file. We'll reopen in sigusr2 handler
-			fclose(log_file);
+			if (log_file)
+				fclose(log_file);
 			log_file = NULL;
 			log_fd = -1;
 			logging_suspended = 1;
@@ -819,20 +861,27 @@ static void do_space_left_action(int admin)
 		case FA_SUSPEND:
 			audit_msg(LOG_ALERT,
 			    "Audit daemon is suspending logging due to low disk space.");
+			// We need to close the file so that manual
+			// intervention can move or delete the file. We
+			// don't want to keep logging to a deleted file.
+			if (log_file)
+				fclose(log_file);
+			log_file = NULL;
+			log_fd = -1;
 			logging_suspended = 1;
 			break;
 		case FA_SINGLE:
-			audit_msg(LOG_ALERT, 
+			audit_msg(LOG_ALERT,
 				"The audit daemon is now changing the system to single user mode");
 			change_runlevel(SINGLE);
 			break;
 		case FA_HALT:
-			audit_msg(LOG_ALERT, 
+			audit_msg(LOG_ALERT,
 				"The audit daemon is now halting the system");
 			change_runlevel(HALT);
 			break;
 		default:
-			audit_msg(LOG_ALERT, 
+			audit_msg(LOG_ALERT,
 			    "Audit daemon is low on disk space for logging and unknown action requested");
 			break;
 	}
@@ -857,7 +906,8 @@ static void do_disk_full_action(void)
 		case FA_EXEC:
 			// Close the logging file in case the script zips or
 			// moves the file. We'll reopen in sigusr2 handler
-			fclose(log_file);
+			if (log_file)
+				fclose(log_file);
 			log_file = NULL;
 			log_fd = -1;
 			logging_suspended = 1;
@@ -866,22 +916,29 @@ static void do_disk_full_action(void)
 		case FA_SUSPEND:
 			audit_msg(LOG_ALERT,
 			    "Audit daemon is suspending logging due to no space left on logging partition.");
+			// We need to close the file so that manual
+			// intervention can move or delete the file. We
+			// don't want to keep logging to a deleted file.
+			if (log_file)
+				fclose(log_file);
+			log_file = NULL;
+			log_fd = -1;
 			logging_suspended = 1;
 			break;
 		case FA_SINGLE:
-			audit_msg(LOG_ALERT, 
+			audit_msg(LOG_ALERT,
 				"The audit daemon is now changing the system to single user mode due to no space left on logging partition");
 			change_runlevel(SINGLE);
 			break;
 		case FA_HALT:
-			audit_msg(LOG_ALERT, 
+			audit_msg(LOG_ALERT,
 				"The audit daemon is now halting the system due to no space left on logging partition");
 			change_runlevel(HALT);
 			break;
 		default:
 			audit_msg(LOG_ALERT, "Unknown disk full action requested");
 			break;
-	} 
+	}
 }
 
 static void do_disk_error_action(const char *func, int err)
@@ -894,7 +951,7 @@ static void do_disk_error_action(const char *func, int err)
 			break;
 		case FA_SYSLOG:
 			if (disk_err_warning < 5) {
-				snprintf(text, sizeof(text), 
+				snprintf(text, sizeof(text),
 			    "%s: Audit daemon detected an error writing an event to disk (%s)",
 					func, strerror(err));
 				audit_msg(LOG_ALERT, "%s", text);
@@ -904,7 +961,8 @@ static void do_disk_error_action(const char *func, int err)
 		case FA_EXEC:
 			// Close the logging file in case the script zips or
 			// moves the file. We'll reopen in sigusr2 handler
-			fclose(log_file);
+			if (log_file)
+				fclose(log_file);
 			log_file = NULL;
 			log_fd = -1;
 			logging_suspended = 1;
@@ -913,28 +971,35 @@ static void do_disk_error_action(const char *func, int err)
 		case FA_SUSPEND:
 			audit_msg(LOG_ALERT,
 			    "Audit daemon is suspending logging due to previously mentioned write error");
+			// We need to close the file so that manual
+			// intervention can move or delete the file. We
+			// don't want to keep logging to a deleted file.
+			if (log_file)
+				fclose(log_file);
+			log_file = NULL;
+			log_fd = -1;
 			logging_suspended = 1;
 			break;
 		case FA_SINGLE:
-			audit_msg(LOG_ALERT, 
+			audit_msg(LOG_ALERT,
 				"The audit daemon is now changing the system to single user mode due to previously mentioned write error");
 			change_runlevel(SINGLE);
 			break;
 		case FA_HALT:
-			audit_msg(LOG_ALERT, 
+			audit_msg(LOG_ALERT,
 				"The audit daemon is now halting the system due to previously mentioned write error.");
 			change_runlevel(HALT);
 			break;
 		default:
-			audit_msg(LOG_ALERT, 
+			audit_msg(LOG_ALERT,
 				"Unknown disk error action requested");
 			break;
-	} 
+	}
 }
 
 static void rotate_logs_now(void)
 {
-	if (config->max_log_size_action == SZ_KEEP_LOGS) 
+	if (config->max_log_size_action == SZ_KEEP_LOGS)
 		shift_logs();
 	else
 		rotate_logs(0, 0);
@@ -952,7 +1017,7 @@ static void check_excess_logs(void)
 	if (config->max_log_size_action != SZ_ROTATE ||
 			config->num_logs < 2)
 		return;
-	
+
 	len = strlen(config->log_file) + 16;
 	name = (char *)malloc(len);
 	if (name == NULL) { /* Not fatal - just messy */
@@ -960,7 +1025,7 @@ static void check_excess_logs(void)
 		return;
 	}
 
-	// We want 1 beyond the normal logs	
+	// We want 1 beyond the normal logs
 	i = config->num_logs;
 	rc = 0;
 	while (rc == 0) {
@@ -1009,7 +1074,7 @@ static void fix_disk_permissions(void)
 
 	free(path);
 }
- 
+
 static void rotate_logs(unsigned int num_logs, unsigned int keep_logs)
 {
 	int rc, i;
@@ -1029,17 +1094,21 @@ static void rotate_logs(unsigned int num_logs, unsigned int keep_logs)
 	/* Close audit file. fchmod and fchown errors are not fatal because we
 	 * already adjusted log file permissions and ownership when opening the
 	 * log file. */
-	if (fchmod(log_fd, config->log_group ? S_IRUSR|S_IRGRP : S_IRUSR) < 0){
-		audit_msg(LOG_WARNING, "Couldn't change permissions while "
+	if (log_fd >= 0) {
+		if (fchmod(log_fd, config->log_group ? S_IRUSR|S_IRGRP :
+			  S_IRUSR) < 0){
+		    audit_msg(LOG_WARNING, "Couldn't change permissions while "
 			"rotating log file (%s)", strerror(errno));
-	}
-	if (fchown(log_fd, 0, config->log_group) < 0) {
-		audit_msg(LOG_WARNING, "Couldn't change ownership while "
+		}
+		if (fchown(log_fd, 0, config->log_group) < 0) {
+		    audit_msg(LOG_WARNING, "Couldn't change ownership while "
 			"rotating log file (%s)", strerror(errno));
+		}
 	}
-	fclose(log_file);
+	if (log_file)
+		fclose(log_file);
 	log_file = NULL;
-	
+
 	/* Rotate */
 	len = strlen(config->log_file) + 16;
 	oldname = (char *)malloc(len);
@@ -1061,7 +1130,7 @@ static void rotate_logs(unsigned int num_logs, unsigned int keep_logs)
 		num_logs = config->num_logs;
 
 	/* Handle this case first since it will not enter the for loop */
-	if (num_logs == 2) 
+	if (num_logs == 2)
 		snprintf(oldname, len, "%s.1", config->log_file);
 
 	known_logs = 0;
@@ -1073,7 +1142,7 @@ static void rotate_logs(unsigned int num_logs, unsigned int keep_logs)
 		if (rc == -1 && errno != ENOENT) {
 			// Likely errors: ENOSPC, ENOMEM, EBUSY
 			int saved_errno = errno;
-			audit_msg(LOG_ERR, 
+			audit_msg(LOG_ERR,
 				"Error rotating logs from %s to %s (%s)",
 				oldname, newname, strerror(errno));
 			if (saved_errno == ENOSPC && fs_space_left == 1) {
@@ -1103,7 +1172,7 @@ static void rotate_logs(unsigned int num_logs, unsigned int keep_logs)
 		/* At this point, we've failed to rotate the original log.
 		 * So, let's make the old log writable and try again next
 		 * time */
-		chmod(config->log_file, 
+		chmod(config->log_file,
 			config->log_group ? S_IWUSR|S_IRUSR|S_IRGRP :
 			S_IWUSR|S_IRUSR);
 	}
@@ -1112,7 +1181,7 @@ static void rotate_logs(unsigned int num_logs, unsigned int keep_logs)
 	/* open new audit file */
 	if (open_audit_log()) {
 		int saved_errno = errno;
-		audit_msg(LOG_CRIT, 
+		audit_msg(LOG_CRIT,
 			"Could not reopen a log after rotating.");
 		logging_suspended = 1;
 		do_disk_error_action("reopen", saved_errno);
@@ -1138,7 +1207,7 @@ static void shift_logs(void)
 	// Find last log
 	num_logs = last_log;
 	while (num_logs) {
-		snprintf(name, len, "%s.%u", config->log_file, 
+		snprintf(name, len, "%s.%u", config->log_file,
 						num_logs);
 		if (access(name, R_OK) != 0)
 			break;
@@ -1151,7 +1220,7 @@ static void shift_logs(void)
 		audit_msg(LOG_WARNING, "Last known log disappeared (%s)", name);
 		num_logs = last_log = 1;
 		while (num_logs) {
-			snprintf(name, len, "%s.%u", config->log_file, 
+			snprintf(name, len, "%s.%u", config->log_file,
 							num_logs);
 			if (access(name, R_OK) != 0)
 				break;
@@ -1176,7 +1245,7 @@ static int open_audit_log(void)
 	if (config->write_logs == 0)
 		return 0;
 
-	flags = O_WRONLY|O_APPEND|O_NOFOLLOW;
+	flags = O_WRONLY|O_APPEND|O_NOFOLLOW|O_CLOEXEC;
 	if (config->flush == FT_DATA)
 		flags |= O_DSYNC;
 	else if (config->flush == FT_SYNC)
@@ -1221,12 +1290,6 @@ retry:
 		}
 	}
 
-	if (fcntl(lfd, F_SETFD, FD_CLOEXEC) == -1) {
-		audit_msg(LOG_ERR, "Error setting log file CLOEXEC flag (%s)",
-			strerror(errno));
-		close(lfd);
-		return 1;
-	}
 	if (fchmod(lfd, config->log_group ? S_IRUSR|S_IWUSR|S_IRGRP :
 							S_IRUSR|S_IWUSR) < 0) {
 		audit_msg(LOG_ERR,
@@ -1265,7 +1328,7 @@ static void change_runlevel(const char *level)
 
 	pid = fork();
 	if (pid < 0) {
-		audit_msg(LOG_ALERT, 
+		audit_msg(LOG_ALERT,
 			"Audit daemon failed to fork switching runlevels");
 		return;
 	}
@@ -1297,7 +1360,7 @@ static void safe_exec(const char *exe)
 
 	pid = fork();
 	if (pid < 0) {
-		audit_msg(LOG_ALERT, 
+		audit_msg(LOG_ALERT,
 			"Audit daemon failed to fork doing safe_exec");
 		return;
 	}
@@ -1355,7 +1418,7 @@ static void reconfigure(struct auditd_event *e)
 		oconf->priority_boost = nconf->priority_boost;
 		errno = 0;
 		rc = nice(-oconf->priority_boost);
-		if (rc == -1 && errno) 
+		if (rc == -1 && errno)
 			audit_msg(LOG_WARNING, "Cannot change priority in "
 					"reconfigure (%s)", strerror(errno));
 	}
@@ -1365,7 +1428,7 @@ static void reconfigure(struct auditd_event *e)
 
 	// Only update this if we are in background mode since
 	// foreground mode writes to stderr.
-	if ((oconf->write_logs != nconf->write_logs) && 
+	if ((oconf->write_logs != nconf->write_logs) &&
 				(oconf->daemonize == D_BACKGROUND)) {
 		oconf->write_logs = nconf->write_logs;
 		need_reopen = 1;
@@ -1385,8 +1448,8 @@ static void reconfigure(struct auditd_event *e)
 		free((void *)nconf->action_mail_acct);
 
 	// node_name
-	if (oconf->node_name_format != nconf->node_name_format || 
-			(oconf->node_name && nconf->node_name && 
+	if (oconf->node_name_format != nconf->node_name_format ||
+			(oconf->node_name && nconf->node_name &&
 			strcmp(oconf->node_name, nconf->node_name) != 0)) {
 		oconf->node_name_format = nconf->node_name_format;
 		free((char *)oconf->node_name);
@@ -1396,7 +1459,7 @@ static void reconfigure(struct auditd_event *e)
 	// network listener
 	auditd_tcp_listen_reconfigure(nconf, oconf);
 
-	// distribute network events	
+	// distribute network events
 	oconf->distribute_network_events = nconf->distribute_network_events;
 
 	// Dispatcher items
@@ -1410,7 +1473,7 @@ static void reconfigure(struct auditd_event *e)
 		oconf->plugin_dir = nconf->plugin_dir;
 	}
 
-	/* At this point we will work on the items that are related to 
+	/* At this point we will work on the items that are related to
 	 * a single log file. */
 
 	// max logfile action
@@ -1446,12 +1509,13 @@ static void reconfigure(struct auditd_event *e)
 		free((void *)nconf->log_file);
 
 	if (need_reopen) {
-		fclose(log_file);
+		if (log_file)
+			fclose(log_file);
 		log_file = NULL;
 		fix_disk_permissions();
 		if (open_audit_log()) {
 			int saved_errno = errno;
-			audit_msg(LOG_ERR, 
+			audit_msg(LOG_ERR,
 				"Could not reopen a log after reconfigure");
 			logging_suspended = 1;
 			// Likely errors: ENOMEM, ENOSPC
@@ -1462,7 +1526,7 @@ static void reconfigure(struct auditd_event *e)
 		}
 	}
 
-	/* At this point we will start working on items that are 
+	/* At this point we will start working on items that are
 	 * related to the amount of space on the partition. */
 
 	// space left
@@ -1577,7 +1641,7 @@ static void reconfigure(struct auditd_event *e)
         }
 
 	e->reply.type = AUDIT_DAEMON_CONFIG;
-	e->reply.len = snprintf(e->reply.msg.data, MAX_AUDIT_MESSAGE_LENGTH-2, 
+	e->reply.len = snprintf(e->reply.msg.data, MAX_AUDIT_MESSAGE_LENGTH-2,
 	"%s: op=reconfigure state=changed auid=%u pid=%d subj=%s res=success",
 		date, uid, pid, ctx );
 	e->reply.message = e->reply.msg.data;
