@@ -1,6 +1,6 @@
 /*
 * ausearch-parse.h - Header file for ausearch-llist.c
-* Copyright (c) 2005 Red Hat Inc., Durham, North Carolina.
+* Copyright (c) 2005,2020 Red Hat
 * All Rights Reserved.
 *
 * This software may be freely redistributed and/or modified under the
@@ -15,19 +15,21 @@
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING. If not, write to the
-* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor 
+* Boston, MA 02110-1335, USA.
 *
 * Authors:
 *   Steve Grubb <sgrubb@redhat.com>
 */
 
-#ifndef AUPARSE_HEADER
-#define AUPARSE_HEADER
+#ifndef AUSEARCH_PARSE_HEADER
+#define AUSEARCH_PARSE_HEADER
 
 #include "config.h"
 #include "ausearch-llist.h"
 
 int extract_search_items(llist *l);
+void lookup_uid_destroy_list(void);
 
 #endif
 
